# pylint: disable-all

try:
    import aiohttp
except ImportError:
    raise RuntimeError(
        "aiohttp is not installed, possibly running in python27?")
from logging import getLogger

logger = getLogger("trustpilot.async_client")

from os import environ
import base64
from trustpilot import auth


class TrustpilotAsyncSession():
    __SUPPORTED_HTTP_METHODS = [
        'post',
        'get',
        'put',
        'delete'
    ]

    def __init__(self, *args, **kwargs):
        self.setup(**kwargs)
        self.headers = {}

    def setup(self, api_host=None, api_key=None, api_secret=None,
              username=None, password=None,
              access_token=None, token_issuer_path=None,
              token_issuer_host=None, **kwargs):

        self.api_host = api_host or environ.get(
            'TRUSTPILOT_API_HOST', 'https://api.trustpilot.com')
        self.token_issuer_host = token_issuer_host or self.api_host
        self.access_token = access_token
        self.token_issuer_path = token_issuer_path or environ.get(
            'TRUSTPILOT_API_TOKEN_ISSUER_PATH', "oauth/oauth-business-users-for-applications/accesstoken")
        self.hooks = dict()

        if not self.api_host.startswith("http"):
            raise aiohttp.http_exceptions.InvalidURLError(
                "'{}' is not a valid api_host url".format(api_host))

        try:
            self.api_key = api_key or environ['TRUSTPILOT_API_KEY']
            self.api_secret = api_secret or environ.get(
                'TRUSTPILOT_API_SECRET', '')
            self.username=username or environ.get('TRUSTPILOT_USERNAME')
            self.password=password or environ.get('TRUSTPILOT_PASSWORD')
            self.access_token = access_token
        except KeyError as e:
            logger.debug(
                "Not auth setup, missing env-var or setup for {}".format(e))

        return self

    async def get_request_auth_headers(self):
        url, data, headers = auth.create_access_token_request_params(self)
        async with aiohttp.ClientSession() as session:
            async with session.post(url, data=data, headers=headers) as response:
                response_json = await response.json()
                self.access_token = response_json["access_token"]
                self.headers.update(dict(
                    Authorization="Bearer {}".format(self.access_token),
                    apikey=self.api_key
                ))

    async def authenticated_request(self, method, url, **kwargs):
        if method not in self.__SUPPORTED_HTTP_METHODS:
            raise RuntimeError("Http method {} not supported".format(method))
        if not any(prefix in url for prefix in ["http://", "https://"]):
            url = "{}{}".format(self.api_host, url)

        async with aiohttp.ClientSession(headers=self.headers) as session:
            http_method = getattr(session, method)
            response = await http_method(url, **kwargs)

        if response.status == 401:
            await self.get_request_auth_headers()
            async with aiohttp.ClientSession(headers=self.headers) as session:
                http_method = getattr(session, method)
                response = await http_method(url, **kwargs)
        return response

    async def post(self, url, *args, **kwargs):
        return await self.authenticated_request('post', url, *args, **kwargs)

    async def get(self, url, *args, **kwargs):

        return await self.authenticated_request('get', url, *args, **kwargs)

    async def put(self, url, *args, **kwargs):
        return await self.authenticated_request('put', url, *args, **kwargs)

    async def delete(self, url, *args, **kwargs):
        return await self.authenticated_request('delete', url, *args, **kwargs)


default_session = TrustpilotAsyncSession()
get = default_session.get
post = default_session.post
put = default_session.put
delete = default_session.delete



import json
from click.testing import CliRunner
try:
    from unittest import mock
except ImportError:
    import mock
import unittest


from trustpilot.cli import cli
from trustpilot import __version__


_creds_list = [
    "--host", "http://hostname",
    "--key", "secret_key",
    "--secret", "secret_secret",
    "--username", "username",
    "--password", "password",
]

class TestCliMethods(unittest.TestCase):
    def setUp(self):
        self.runner = CliRunner()
        response_mock = mock.Mock()
        response_mock.url = "https://api.trustpilot.com/v1/business-units/5400267300006400057a0113/reviews"
        response_mock.status_code = 401
        response_mock.headers = {
                "Access-Control-Allow-Headers": "Authorization, Accept, Accept-Charset, Accept-Encoding, Accept-Language, Cache-Control, Connection, Content-Length, Content-Type, Host, Origin, User-Agent, ApiKey, X-Requested-With",
                "Access-Control-Allow-Methods": "GET",
                "Access-Control-Allow-Origin": "*",
                "Access-Control-Max-Age": "3628800",
                "Content-Type": "application/json",
                "Date": "Mon, 30 Jan 2017 08:59:49 GMT",
                "Server": "Apigee Router",
                "Content-Length": "90",
                "Connection": "keep-alive"
            }
        response_mock.json.return_value = {
                "fault": {
                    "faultstring": "Invalid ApiKey",
                    "detail": {
                        "errorcode": "oauth.v2.InvalidApiKey"
                    }
                }
            }
        self.response_mock = response_mock
        self.expected_output = u'''{
          "url": "https://api.trustpilot.com/v1/business-units/5400267300006400057a0113/reviews",
          "status": 401,
          "content": {
            "fault": {
              "faultstring": "Invalid ApiKey",
              "detail": {
                "errorcode": "oauth.v2.InvalidApiKey"
              }
            }
          }
        }
        '''
        self.expected_verbose_output = u'''{
          "url": "https://api.trustpilot.com/v1/business-units/5400267300006400057a0113/reviews",
          "status": 401,
          "headers": {
            "Content-Length": "90",
            "Access-Control-Allow-Methods": "GET",
            "Access-Control-Max-Age": "3628800",
            "Date": "Mon, 30 Jan 2017 08:59:49 GMT",
            "Access-Control-Allow-Origin": "*",
            "Access-Control-Allow-Headers": "Authorization, Accept, Accept-Charset, Accept-Encoding, Accept-Language, Cache-Control, Connection, Content-Length, Content-Type, Host, Origin, User-Agent, ApiKey, X-Requested-With",
            "Content-Type": "application/json",
            "Connection": "keep-alive",
            "Server": "Apigee Router"
          },
          "content": {
            "fault": {
              "faultstring": "Invalid ApiKey",
              "detail": {
                "errorcode": "oauth.v2.InvalidApiKey"
              }
            }
          }
        }
        '''

    def assert_output_equal(self, output, expected_output):
        output = "".join(output.split())
        expected_output = "".join(expected_output.split())
        try:
            output = json.loads(output)
            expected_output = json.loads(expected_output)
        except ValueError:
            pass
        assert  output == expected_output

    @mock.patch("trustpilot.cli.client", autospec=True)
    def test_create_access_token(self, client_mock):
        client_mock.default_session.access_token = "access_token"
        result = self.runner.invoke(cli, _creds_list + ["create_access_token"])
        self.assert_output_equal(result.output, "access_token")

    @mock.patch("trustpilot.cli.client", autospec=True)
    @mock.patch("trustpilot.cli.auth")
    def test_no_verbosity_with_get(self, auth_mock, client_mock):

        client_mock.get.return_value = self.response_mock

        result = self.runner.invoke(cli, _creds_list + [
            "get", "/v1/business-units/5400267300006400057a0113/reviews"])

        self.assert_output_equal(result.output, self.expected_output)

    @mock.patch("trustpilot.cli.client", autospec=True)
    @mock.patch("trustpilot.cli.auth")
    def test_low_verbosity_with_get(self, auth_mock, client_mock):
        client_mock.get.return_value = self.response_mock

        result = self.runner.invoke(cli, _creds_list + [
            "-v",
            "get",
            "/v1/business-units/5400267300006400057a0113/reviews"
        ])
        self.assert_output_equal(result.output, self.expected_verbose_output)


    @mock.patch("trustpilot.cli.client", autospec=True)
    @mock.patch("trustpilot.cli.auth")
    def test_no_verbosity_with_post(self, auth_mock, client_mock):
        client_mock.post.return_value = self.response_mock

        result = self.runner.invoke(cli, _creds_list + [
            "post",
            "/v1/business-units/5400267300006400057a0113/reviews",
            "--data",
            '{"foo": "bar"}',
            "--content_type", "application/json"])
        self.assert_output_equal(result.output, self.expected_output)


    @mock.patch("trustpilot.cli.client", autospec=True)
    @mock.patch("trustpilot.cli.auth")
    def test_no_verbosity_with_put(self, auth_mock, client_mock):
        client_mock.put.return_value = self.response_mock

        result = self.runner.invoke(cli, _creds_list + [
            "put",
            "/v1/business-units/5400267300006400057a0113/reviews",
            "--data",
            '{"foo": "bar"}',
            "--content_type", "application/json"])
        self.assert_output_equal(result.output, self.expected_output)


    @mock.patch("trustpilot.cli.client", autospec=True)
    @mock.patch("trustpilot.cli.auth")
    def test_no_verbosity_with_delete(self, auth_mock, client_mock):
        client_mock.delete.return_value = self.response_mock

        result = self.runner.invoke(cli, _creds_list + [
            "delete", "/v1/business-units/5400267300006400057a0113/reviews"])

        self.assert_output_equal(result.output, self.expected_output)



import requests
import base64
import logging
import platform
from trustpilot import __version__ as VERSION


OS = platform.system()
PYTHON_VERSION = platform.python_version()
SCOPE = 'external'


def get_user_agent():
    user_agent = "python-trustpilot-client?scope={scope}&version={version}&python-version={python_version}&os={os}".format(
        scope=SCOPE,
        version=VERSION,
        python_version=PYTHON_VERSION,
        os=OS
    )
    return user_agent


def create_access_token_request_params(session):
    url = "{token_issuer_host}/v1/{token_issuer_path}".format(
        token_issuer_host=session.token_issuer_host,
        token_issuer_path=session.token_issuer_path
    )
    data = {
        "grant_type": "password",
        "username": session.username,
        "password": session.password
    }
    
    headers = {
        "Authorization": "Basic {}".format(base64.b64encode(
            (session.api_key + ":" + session.api_secret
             ).encode("ascii")).decode("ascii")
        ),
        'User-Agent': get_user_agent()
    }

    return url, data, headers



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup
import sys
import os

try:
    from pypandoc import convert
    read_md = lambda f: convert(f, 'rst')
except ImportError:
    print("warning: pypandoc module not found, could not convert Markdown to RST")
    read_md = lambda f: open(f, 'r').read()

readme = read_md('README.md')

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'Click==6.7',
    'requests==2.20.0',
]

test_requirements = [
    'responses==0.6',
    'mock',
    'pytest==3.2.0',
    'pytest-readme==1.0.0'
] + requirements

if sys.version_info >= (3,5):
    test_requirements.append('aioresponses==0.3.1')

async_requirements = [
    'aiohttp==2.3.9'
] + requirements

extras = {
    'test' : test_requirements,
    'async' : async_requirements
}
# get version
metadata = {}
version_filename = os.path.join(os.path.dirname(__file__), 'trustpilot','__init__.py')
exec(open(version_filename).read(), None, metadata)

setup(
    name='trustpilot',
    version=metadata['__version__'],
    description="trustpilot api client including cli tool",
    long_description=readme + '\n\n' + history,
    author="jgv",
    author_email='jgv@trustpilot.com',
    url='https://github.com/trustpilot/python-trustpilot',
    packages=[
        'trustpilot',
    ],
    package_dir={'trustpilot':
                 'trustpilot'},
    entry_points={
        'console_scripts': [
            'trustpilot_api_client=trustpilot.cli:cli'
        ]
    },
    include_package_data=True,
    install_requires=requirements,
    zip_safe=False,
    keywords='trustpilot api client',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'Natural Language :: English',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    test_suite='tests',
    tests_require=test_requirements,
    extras_require=extras
)



# pylint: disable-all
import pytest
import sys

@pytest.mark.skipif(sys.version_info < (3, 5, 2),
                                reason="requires python 3.5.2 or above")
def test_async_client_auth_and_get():
    import os
    from aioresponses import aioresponses
    import asyncio
    from trustpilot import async_client

    loop = asyncio.get_event_loop()

    with aioresponses() as m:
        m.get(
            'https://api.tp-staging.com/v1/foo/bar',
            status=401
        )
        m.post(
            'https://api.tp-staging.com/v1/oauth/oauth-business-users-for-applications/accesstoken',
            payload=dict(
                access_token="foobarbaz"
            )
        )
        m.get(
            'https://api.tp-staging.com/v1/foo/bar',
            payload=dict(
                foo='foobarbaz'
            )
        )

        session = async_client.TrustpilotAsyncSession(
            api_host='https://api.tp-staging.com',
            api_key='something',
            api_secret='secret',
            username='username',
            password='password'
        )

        async def get_response():
            response = await session.get('/v1/foo/bar')
            response_json = await response.json()
            assert response_json['foo'] == 'foobarbaz'
            assert session.access_token == 'foobarbaz'

        resp = loop.run_until_complete(get_response())
